/*
 * A registry of objects supporting paging for files.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <map>
#include <mutex>

#include <fsserver/file_opening.h>
#include <fsserver/notification.h>
#include <fsserver/page_mapper.h>
#include <fsserver/pager.h>
#include <fsserver/pages.h>



/* Mapping type from file identifiers to page mappers. */

typedef std::map<fileid_t, PageMapper *> FileMapping;
typedef std::pair<fileid_t, PageMapper *> FileMappingEntry;



/* A registry of mappers for accessors. */

class FilePaging : public NotificationSupport
{
protected:
    Pages *_pages;
    FileMapping _mappers;
    std::mutex _lock;

    /* Mapper registry access. */

    PageMapper *get(fileid_t fileid);

    void remove(fileid_t fileid, PageMapper *mapper);

    void set(fileid_t fileid, PageMapper *mapper);

    /* Pager initialisation methods. */

    map_flags_t get_flags(flags_t flags);

public:
    explicit FilePaging(Pages *pages);

    /* Pager initialisation methods. */

    long get_mapper(FileOpening *opening, const char *path, flags_t flags, fileid_t fileid, PageMapper **mapper);

    long get_pager(FileOpening *opening, const char *path, flags_t flags, Pager **pager);

    /* Methods for the pager. */

    void detach_pager(fileid_t fileid, PageMapper *mapper);
};

// vim: tabstop=4 expandtab shiftwidth=4
