/*
 * An opener for a test file containing generated content.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "test_file_accessor.h"
#include "test_file_opener.h"

#include <stdlib.h>

/* Support for providing access to files. */

TestFileOpener::TestFileOpener(FilePaging *paging, offset_t file_size)
: OpenerResource(paging), _file_size(file_size)
{
}

/* Return a file identifier for the given 'path'. */

fileid_t TestFileOpener::get_fileid(const char *path, flags_t flags)
{
    (void) flags;

    /* NOTE: Just convert the path to a number. */

    return atol(path);
}

/* Return a new accessor for 'fileid'. */

long TestFileOpener::make_accessor(const char *path, flags_t flags, fileid_t fileid, Accessor **accessor)
{
    (void) flags; (void) path;
    *accessor = new TestFileAccessor(fileid, _file_size);
    return L4_EOK;
}

// vim: tabstop=4 expandtab shiftwidth=4
